IPC_FIFO_NAME = 'IPC_PIPE_DEFAULT'
IPC_FIFO_NAME1 = 'IPC_PIPE_CUSTOM1'
IPC_FIFO_NAME2 = 'IPC_PIPE_CUSTOM2'
IPC_FIFO_NAME3 = 'IPC_PIPE_CUSTOM3'
IPC_FIFO_NAME4 = 'IPC_PIPE_CUSTOM4'
IPC_FIFO_NAME5 = 'IPC_PIPE_CUSTOM5'

